@@ -8,6 +8,9 @@ import datetime
 import logging
 import subprocess
 from collections import defaultdict
+import pprint
+import fnmatch
+import concurrent.futures
 
 import six
 import pytest
@@ -20,7 +23,7 @@ LOGGER = logging.getLogger("elk-reporter")
 
 def pytest_runtest_makereport(item, call):
     report = _makereport(item, call)
-    report.keywords = dict(item.keywords)
+    report.keywords = list([m.name for m in item.keywords.get("pytestmark", [])])
     return report
 
 
@@ -31,7 +34,7 @@ def pytest_addoption(parser):
         action="store",
         dest="es_address",
         default=None,
-        help="Elasticsearch addresss",
+        help="Elasticsearch address",
     )
 
     group.addoption(
@@ -58,7 +61,32 @@ def pytest_addoption(parser):
         help="Elasticsearch connection timeout",
     )
 
-    parser.addini("es_address", help="Elasticsearch addresss", default=None)
+    group.addoption(
+        "--es-slices",
+        action="store_true",
+        dest="es_slices",
+        default=False,
+        help="Splice collected tests base on history data",
+    )
+
+    group.addoption(
+        "--es-max-splice-time",
+        action="store",
+        type=float,
+        dest="es_max_splice_time",
+        default=60,
+        help="Max duration of each splice, in minutes",
+    )
+    group.addoption(
+        "--es-default-test-time",
+        action="store",
+        type=float,
+        dest="es_default_test_time",
+        default=120,
+        help="Default time for a test, if history isn't found for it, in seconds",
+    )
+
+    parser.addini("es_address", help="Elasticsearch address", default=None)
     parser.addini("es_username", help="Elasticsearch username", default=None)
     parser.addini("es_password", help="Elasticsearch password", default=None)
     parser.addini(
@@ -88,10 +116,18 @@ def get_username():
     try:
         return getpass.getuser()
     except ImportError:
-        # seem like our friends in AppVeyor took security into a new level,
-        # and no username env vars causes getpass to go crazy
-        # https://github.com/python-cmd2/cmd2/pull/372/files#diff-ed451f9960c50a6a096b11155fdbfc1dR325
-        return "AppVeyorIt'sWeirdNotToHaveUserName"
+        try:
+            return os.getlogin()
+        except Exception:  # pylint: disable=broad-except
+            # seems like there are case we can't get the name of the user that is currently running
+            LOGGER.warning(
+                "couldn't figure out which user is currently running setting to 'unknown'"
+            )
+            LOGGER.warning(
+                "see https://docs.python.org/3/library/getpass.html#getpass.getuser, "
+                "if you want to configure it correctly"
+            )
+            return "unknown"
 
 
 class ElkReporter(object):  # pylint: disable=too-many-instance-attributes
@@ -106,6 +142,11 @@ class ElkReporter(object):  # pylint: disable=too-many-instance-attributes
         self.es_index_name = config.getini("es_index_name")
         self.es_timeout = config.getoption("es_timeout")
 
+        self.es_max_splice_time = config.getoption("es_max_splice_time")
+        self.es_default_test_time = config.getoption("es_default_test_time")
+
+        self.slices_query_fmt = '(name:"{}") AND (outcome: passed)'
+
         self.stats = dict.fromkeys(
             [
                 "error",
@@ -117,6 +158,7 @@ class ElkReporter(object):  # pylint: disable=too-many-instance-attributes
                 "passed & error",
                 "failure & error",
                 "skipped & error",
+                "error & error",
             ],
             0,
         )
@@ -184,9 +226,9 @@ class ElkReporter(object):  # pylint: disable=too-many-instance-attributes
 
         if report.when == "teardown":
             old_report = self.get_report(report)
-            if report.passed:
+            if report.passed and old_report:
                 self.report_test(old_report[0], old_report[1])
-            if report.failed:
+            if report.failed and old_report:
                 self.report_test(
                     report, old_report[1] + " & error", old_report=old_report[0]
                 )
@@ -196,12 +238,13 @@ class ElkReporter(object):  # pylint: disable=too-many-instance-attributes
     def report_test(self, item_report, outcome, old_report=None):
         self.stats[outcome] += 1
         test_data = dict(
+            item_report.user_properties,
             timestamp=datetime.datetime.utcnow().isoformat(),
             name=item_report.nodeid,
             outcome=outcome,
             duration=item_report.duration,
-            markers=list([m.name for m in item_report.keywords.get("pytestmark", [])]),
-            **self.session_data
+            markers=item_report.keywords,
+            **self.session_data,
         )
         test_data.update(self.test_data[item_report.nodeid])
         del self.test_data[item_report.nodeid]
@@ -225,7 +268,7 @@ class ElkReporter(object):  # pylint: disable=too-many-instance-attributes
     def pytest_terminal_summary(self, terminalreporter):
         verbose = terminalreporter.config.getvalue("verbose")
 
-        if not self.config.getoption("collectonly") and verbose < 2:
+        if not self.config.getoption("collectonly") and verbose < 2 and self.es_address:
             terminalreporter.write_sep(
                 "-",
                 "stats posted to elasticsearch [%s]: %s"
@@ -237,7 +280,7 @@ class ElkReporter(object):  # pylint: disable=too-many-instance-attributes
             timestamp=datetime.datetime.utcnow().isoformat(),
             outcome="internal-error",
             faiure_message=excrepr,
-            **self.session_data
+            **self.session_data,
         )
         self.post_to_elasticsearch(test_data)
 
@@ -252,6 +295,131 @@ class ElkReporter(object):  # pylint: disable=too-many-instance-attributes
             except Exception as ex:  # pylint: disable=broad-except
                 LOGGER.warning("Failed to POST to elasticsearch: [%s]", str(ex))
 
+    def fetch_test_duration(
+        self, collected_test_list, default_time_sec=120.0, max_workers=20
+    ):
+        """
+        fetch test 95 percentile duration of a list of tests
+
+        :param collected_test_list: the names of the test to lookup
+        :param default_time_sec: the time to return when no history data found
+        :param max_workers: number of threads to use for concurrency
+
+        :returns: map from test_id to 95 percentile duration
+        """
+
+        test_durations = []
+        session = requests.Session()
+
+        def get_test_stats(test_id):
+            url = "{0.es_url}/{0.es_index_name}/_search?size=0".format(self)
+            body = {
+                "query": {
+                    "query_string": {"query": self.slices_query_fmt.format(test_id)}
+                },
+                "aggs": {
+                    "percentiles_duration": {
+                        "percentiles": {"field": "duration", "percents": [90, 95, 99]}
+                    },
+                },
+            }
+            try:
+                res = session.post(
+                    url, json=body, auth=self.es_auth, timeout=self.es_timeout
+                )
+                res.raise_for_status()
+                return dict(
+                    test_name=test_id,
+                    duration=res.json()["aggregations"]["percentiles_duration"][
+                        "values"
+                    ]["95.0"],
+                )
+            except (requests.exceptions.ReadTimeout, requests.exceptions.HTTPError):
+                return dict(test_name=test_id, duration=None)
+
+        with concurrent.futures.ThreadPoolExecutor(max_workers=max_workers) as executor:
+            future_to_test_id = {
+                executor.submit(get_test_stats, test_id): test_id
+                for test_id in collected_test_list
+            }
+            for future in concurrent.futures.as_completed(future_to_test_id):
+                test_id = future_to_test_id[future]
+                try:
+                    test_durations.append(future.result())
+                except Exception:  # pylint: disable=broad-except
+                    LOGGER.exception("'%s' generated an exception", test_id)
+
+        for test in test_durations:
+            if not test["duration"]:
+                test["duration"] = default_time_sec
+        test_durations.sort(key=lambda x: x["duration"])
+        LOGGER.debug(pprint.pformat(test_durations))
+
+        return test_durations
+
+    @staticmethod
+    def clear_old_exclude_files(outputdir):
+        print("clear old exclude files")
+        # Get a list of all files in directory
+        for root_dir, _, filenames in os.walk(outputdir):
+            # Find the files that matches the given pattern
+            for filename in fnmatch.filter(filenames, "include_*.txt"):
+                try:
+                    os.remove(os.path.join(root_dir, filename))
+                except OSError:
+                    print(f"Error while deleting file {filename}")
+
+    @staticmethod
+    def split_files_test_list(outputdir, slices):
+        for i, current_slice in enumerate(slices):
+            print(
+                f"{i}: {datetime.timedelta(0, current_slice['total'])} "
+                f"- {len(current_slice['tests'])} - {current_slice['tests']}"
+            )
+            include_filename = os.path.join(outputdir, "include_%03d.txt" % i)
+
+            with open(include_filename, "w") as slice_file:
+                for case in current_slice["tests"]:
+                    slice_file.write(case + "\n")
+
+    @staticmethod
+    def make_test_slices(test_data, max_slice_duration):
+        slices = []
+        while test_data:
+            current_test = test_data.pop(0)
+            for current_slice in slices:
+                if (
+                    current_slice["total"] + float(current_test["duration"])
+                    > max_slice_duration
+                ):
+                    continue
+                current_slice["total"] += float(current_test["duration"])
+                current_slice["tests"] += [current_test["test_name"]]
+                break
+            else:
+                slices += [dict(total=0.0, tests=[])]
+                current_slice = slices[-1]
+                current_slice["total"] += float(current_test["duration"])
+                current_slice["tests"] += [current_test["test_name"]]
+        return slices
+
+    def pytest_collection_finish(self, session):
+
+        if self.config.getoption("es_slices"):
+            assert (
+                self.es_default_test_time and self.es_max_splice_time
+            ), "'--es-max-splice-time' and '--es-default-test-time' should be positive numbers"
+            test_history_data = self.fetch_test_duration(
+                [item.nodeid.replace("::()", "") for item in session.items],
+                default_time_sec=self.es_default_test_time,
+            )
+            slices = self.make_test_slices(
+                test_history_data, max_slice_duration=self.es_max_splice_time * 60
+            )
+            LOGGER.debug(pprint.pformat(slices))
+            self.clear_old_exclude_files(outputdir=".")
+            self.split_files_test_list(outputdir=".", slices=slices)
+
 
 @pytest.fixture(scope="session")
 def elk_reporter(request):
@@ -301,6 +469,20 @@ def travis_data(request):
         elk.session_data.update(**travis_env)
 
 
+@pytest.fixture(scope="session", autouse=True)
+def github_data(request):
+    """
+    Append github ci job and user data into results session
+    """
+    if os.environ.get("GITHUB_ACTIONS", False) == "true":
+        github_env = {
+            k.lower(): v for k, v in os.environ.items() if k.startswith("GITHUB_")
+        }
+
+        elk = request.config.pluginmanager.get_plugin("elk-reporter-runtime")
+        elk.session_data.update(**github_env)
+
+
 @pytest.fixture(scope="session", autouse=True)
 def git_data(request):
     """
@@ -318,7 +500,9 @@ def git_data(request):
     for key, command in cmds:
         try:
             git_info[key] = (
-                subprocess.check_output(command, shell=True).decode("utf-8").strip()
+                subprocess.check_output(command, shell=True, stderr=subprocess.DEVNULL)
+                .decode("utf-8")
+                .strip()
             )
         except subprocess.CalledProcessError:
             pass
