@@ -1,44 +1,9 @@
 # -*- coding: utf-8 -*-
 
 import os
-import re
 import json
 
 import pytest
-import requests_mock as rm_module
-
-try:
-    from distutils import version  # pylint: disable=no-name-in-module
-
-    _PYTEST_VERSION = version.StrictVersion(pytest.__version__)
-    _PYTEST_30 = _PYTEST_VERSION >= version.StrictVersion("3.0.0")
-except Exception:  # pylint: disable=broad-except
-    _PYTEST_30 = False
-
-if _PYTEST_30:
-    _FIXTRUE_TYPE = pytest.fixture
-else:
-    _FIXTRUE_TYPE = pytest.yield_fixture
-
-
-@_FIXTRUE_TYPE(scope="function")  # executed on every test
-def requests_mock():
-    """Mock out the requests component of your code with defined responses.
-    Mocks out any requests made through the python requests library with useful
-    responses for unit testing. See:
-    https://requests-mock.readthedocs.io/en/latest/
-    """
-    kwargs = {"real_http": False}
-
-    with rm_module.Mocker(**kwargs) as mock:
-        yield mock
-
-
-@pytest.fixture(scope="function", autouse=True)
-def configure_es_mock(requests_mock):  # pylint: disable=redefined-outer-name
-    matcher = re.compile(r"http://127.0.0.1:9200/.*")
-    # requests_mock.post(matcher, text="data", status_code=201, real_http=True)
-    requests_mock.post(matcher, real_http=True)
 
 
 def test_failures(testdir, requests_mock):  # pylint: disable=redefined-outer-name
@@ -136,9 +101,11 @@ def test_failures(testdir, requests_mock):  # pylint: disable=redefined-outer-na
         u"skipped": 3,
         u"xfailed": 1,
         u"xpass": 1,
+        u"error & error": 0,
     }
 
 
+@pytest.mark.this_marker
 def test_es_configuration(testdir):
     """Make sure that pytest accepts our elasticsearch configuration."""
 
@@ -254,6 +221,26 @@ def test_circle_env_collection(testdir):
     assert result.ret == 0
 
 
+def test_github_env_collection(testdir):
+    os.environ["GITHUB_ACTIONS"] = "true"
+    os.environ["GITHUB_ACTOR"] = "Israel Fruchter"
+
+    # create a temporary pytest test module
+    testdir.makepyfile(
+        """
+        def test_collection_elk(elk_reporter):
+            assert elk_reporter.session_data['github_actor'] == 'Israel Fruchter'
+        """
+    )
+
+    result = testdir.runpytest("--es-address=127.0.0.1:9200", "-v", "-s")
+
+    # fnmatch_lines does an assertion internally
+    result.stdout.fnmatch_lines(["*::test_collection_elk PASSED*"])
+    # make sure that that we get a '0' exit code for the testsuite
+    assert result.ret == 0
+
+
 def test_setup_es_from_code(testdir):
     # create a temporary pytest test module
     testdir.makepyfile(
@@ -404,3 +391,60 @@ def test_marker_collection(
 
     second_report = json.loads(requests_mock.request_history[1].text)
     assert "mark2" in second_report["markers"]
+
+
+def test_xdist(testdir, requests_mock):  # pylint: disable=redefined-outer-name
+    # create a temporary pytest test module
+    testdir.makepyfile(
+        """
+        import pytest
+        @pytest.mark.mark1
+        def test_1(request):
+            pass
+
+        @pytest.mark.mark2
+        def test_2():
+            pass
+        """
+    )
+
+    result = testdir.runpytest("--es-address=127.0.0.1:9200", "-v", "-s", "-n", "2")
+
+    # fnmatch_lines does an assertion internally
+    result.stdout.fnmatch_lines(["*PASSED*test_xdist.py::test_1 "])
+    result.stdout.fnmatch_lines(["*PASSED*test_xdist.py::test_2 "])
+
+    for data in requests_mock.request_history[:2]:
+        report = json.loads(data.text)
+
+        if report["name"] == "test_xdist.py::test_1":
+            assert "mark1" in report["markers"]
+        if report["name"] == "test_xdist.py::test_2":
+            assert "mark2" in report["markers"]
+
+
+def test_user_properties(
+    testdir, requests_mock
+):  # pylint: disable=redefined-outer-name
+    # create a temporary pytest test module
+    testdir.makepyfile(
+        """
+        import pytest
+
+        def test_1(record_property):
+            record_property("example_key", 1)
+            pass
+        """
+    )
+
+    result = testdir.runpytest("--es-address=127.0.0.1:9200", "-v", "-s")
+
+    # fnmatch_lines does an assertion internally
+    result.stdout.fnmatch_lines(["*::test_1 PASSED*"])
+
+    # make sure that that we get a '0' exit code for the testsuite
+    assert result.ret == 0
+
+    report = json.loads(requests_mock.request_history[0].text)
+    assert "example_key" in report
+    assert report["example_key"] == 1
